package javax.tools;

import java.net.MalformedURLException;
import java.net.URLClassLoader;
import java.net.URL;
import java.io.File;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.logging.Logger;
import java.util.Locale;
import java.util.logging.Level;
import java.lang.ref.Reference;
import java.util.Map;

public class ToolProvider
{
    private static final String propertyName = "sun.tools.ToolProvider";
    private static final String loggerName = "javax.tools";
    private static final String defaultJavaCompilerName = "com.sun.tools.javac.api.JavacTool";
    private static final String defaultDocumentationToolName = "com.sun.tools.javadoc.api.JavadocTool";
    private static ToolProvider instance;
    private Map<String, Reference<Class<?>>> toolClasses;
    private Reference<ClassLoader> refToolClassLoader;
    private static final String[] defaultToolsLocation;
    
    static <T> T trace(Level level, Object obj) {
        try {
            if (System.getProperty("sun.tools.ToolProvider") != null) {
                StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
                String format = "???";
                String s = ToolProvider.class.getName();
                if (stackTrace.length > 2) {
                    StackTraceElement stackTraceElement = stackTrace[2];
                    format = String.format((Locale)null, "%s(%s:%s)", new Object[] { stackTraceElement.getMethodName(), stackTraceElement.getFileName(), Integer.valueOf(stackTraceElement.getLineNumber()) });
                    s = stackTraceElement.getClassName();
                }
                Logger logger = Logger.getLogger("javax.tools");
                if ((obj instanceof Throwable)) {
                    logger.logp(level, s, format, obj.getClass().getName(), (Throwable)obj);
                }
                else {
                    logger.logp(level, s, format, String.valueOf(obj));
                }
            }
        }
        catch (SecurityException ex) {
            System.err.format((Locale)null, "%s: %s; %s%n", new Object[] { ToolProvider.class.getName(), obj, ex.getLocalizedMessage() });
        }
        return null;
    }
    
    public static JavaCompiler getSystemJavaCompiler() {
        return (JavaCompiler)instance().getSystemTool(JavaCompiler.class, "com.sun.tools.javac.api.JavacTool");
    }
    
    public static DocumentationTool getSystemDocumentationTool() {
        return (DocumentationTool)instance().getSystemTool(DocumentationTool.class, "com.sun.tools.javadoc.api.JavadocTool");
    }
    
    public static ClassLoader getSystemToolClassLoader() {
        try {
            return instance().getSystemToolClass(JavaCompiler.class, "com.sun.tools.javac.api.JavacTool").getClassLoader();
        }
        catch (Throwable t) {
            return (ClassLoader)trace(Level.WARNING, t);
        }
    }
    
    private static synchronized ToolProvider instance() {
        if (ToolProvider.instance == null) {
            ToolProvider.instance = new ToolProvider();
        }
        return ToolProvider.instance;
    }
    
    private ToolProvider() {
        this.toolClasses = new HashMap();
        this.refToolClassLoader = null;
    }
    
    private <T> T getSystemTool(Class<T> clazz, String s) {
        Class<Object> systemToolClass = this.getSystemToolClass(clazz, s);
        try {
            return systemToolClass.asSubclass(clazz).newInstance();
        }
        catch (Throwable t) {
            trace(Level.WARNING, t);
            return null;
        }
    }
    
    private <T> Class<? extends T> getSystemToolClass(Class<T> clazz, String s) {
        Reference<Object> reference = (Reference)this.toolClasses.get(s);
        Class<Object> systemToolClass = (reference == null) ? null : ((Class)reference.get());
        if (systemToolClass == null) {
            try {
                systemToolClass = this.findSystemToolClass(s);
            }
            catch (Throwable t) {
                return (Class)trace(Level.WARNING, t);
            }
            this.toolClasses.put(s, new WeakReference(systemToolClass));
        }
        return systemToolClass.asSubclass(clazz);
    }
    
    private Class<?> findSystemToolClass(String s) throws MalformedURLException, ClassNotFoundException {
        try {
            return Class.forName(s, false, null);
        }
        catch (ClassNotFoundException ex) {
            trace(Level.FINE, ex);
            ClassLoader instance = (this.refToolClassLoader == null) ? null : ((ClassLoader)this.refToolClassLoader.get());
            if (instance == null) {
                File parentFile = new File(System.getProperty("java.home"));
                if (parentFile.getName().equalsIgnoreCase("jre")) {
                    parentFile = parentFile.getParentFile();
                }
                String[] defaultToolsLocation = ToolProvider.defaultToolsLocation;
                for (int length = defaultToolsLocation.length, i = 0; i < length; ++i) {
                    parentFile = new File(parentFile, defaultToolsLocation[i]);
                }
                if (!parentFile.exists()) {
                    throw ex;
                }
                URL[] urls = { parentFile.toURI().toURL() };
                trace(Level.FINE, urls[0].toString());
                instance = URLClassLoader.newInstance(urls);
                this.refToolClassLoader = new WeakReference(instance);
            }
            return Class.forName(s, false, instance);
        }
    }
    
    static {
        defaultToolsLocation = new String[] { "lib", "tools.jar" };
    }
}
